#include "../pa_http.h"

WebRequest::WebRequest(const QUrl &url, const QString &filePath)
{
    connect(this,SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)),this,SLOT(slot_autenticationRequired(QNetworkReply*,QAuthenticator*)));
    connect(this,SIGNAL(finished(QNetworkReply*)),this,SLOT(slot_finished(QNetworkReply*)));
    connect(this,SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),this,SLOT(slot_proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
    connect(this,SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),this,SLOT(slot_sslErrors(QNetworkReply*,QList<QSslError>)));

    respFile = new QFile();
    QFile::remove(filePath);
    respFile->setFileName(filePath);
    request.setUrl(url);
    if (url.scheme().toLower() == "https")
        request.setSslConfiguration(QSslConfiguration::defaultConfiguration());
    path = QUrl::toPercentEncoding(url.path(),"!$&'()*+,;=:@/");
}

WebRequest::~WebRequest()
{
}

void WebRequest::slot_autenticationRequired(QNetworkReply *reply, QAuthenticator *aut)
{
    Q_UNUSED(reply);
    Q_UNUSED(aut);
}

void WebRequest::slot_finished(QNetworkReply *reply)
{   
    QByteArray data;
    data = reply->readAll();    
    WebReply rpl;    
    rpl.data = data;
    rpl.attributes.connectionEncrypted = reply->attribute(QNetworkRequest::ConnectionEncryptedAttribute).toBool();
    rpl.attributes.reasonPhrase = reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toByteArray();
    rpl.attributes.redirectionTarget = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
    rpl.attributes.sourceIsFromCache = reply->attribute(QNetworkRequest::SourceIsFromCacheAttribute).toBool();
    rpl.attributes.statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
    rpl.attributes.error = reply->error();
    rpl.attributes.errorString = reply->errorString();
    foreach(const QByteArray &h, reply->rawHeaderList())
    {
        rpl.attributes.rawHeaders.insert(h,reply->rawHeader(h));
    }
    if (respFile)
    {
        respFile->open(QIODevice::ReadWrite);
        respFile->write(data);
        respFile->close();
        rpl.filePath = respFile->fileName();
        respFile->deleteLater();
    }
    if (rpl.attributes.statusCode == 0 ||
        reply->error() == QNetworkReply::HostNotFoundError ||
        reply->error() == QNetworkReply::ConnectionRefusedError ||
        reply->error() == QNetworkReply::TimeoutError )
    {
        emit offline();
    }
    else
    {
        emit done(rpl);
    }
    deleteLater();
}

void WebRequest::slot_proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *aut)
{
    Q_UNUSED(proxy);
    Q_UNUSED(aut);
}

void WebRequest::slot_sslErrors(QNetworkReply *reply, const QList<QSslError> &errors)
{    
    Q_UNUSED(reply);
    Q_UNUSED(errors);
}

void WebRequest::post()
{
    QNetworkReply *reply = QNetworkAccessManager::post(request,reqData);
    connect(reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(reply_downloadProgress(qint64,qint64)));
    connect(reply,SIGNAL(error(QNetworkReply::NetworkError)),this,SLOT(reply_error(QNetworkReply::NetworkError)));
    connect(reply,SIGNAL(metaDataChanged()),this,SLOT(reply_metaDataChanged()));
    connect(reply,SIGNAL(sslErrors(QList<QSslError>)),this,SLOT(reply_sslErrors(QList<QSslError>)));
    connect(reply,SIGNAL(uploadProgress(qint64,qint64)),this,SLOT(reply_uploadProgress(qint64,qint64)));
}

void WebRequest::get()
{    
    QNetworkReply *reply = QNetworkAccessManager::get(request);
    connect(reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(reply_downloadProgress(qint64,qint64)));
    connect(reply,SIGNAL(error(QNetworkReply::NetworkError)),this,SLOT(reply_error(QNetworkReply::NetworkError)));
    connect(reply,SIGNAL(metaDataChanged()),this,SLOT(reply_metaDataChanged()));
    connect(reply,SIGNAL(sslErrors(QList<QSslError>)),this,SLOT(reply_sslErrors(QList<QSslError>)));
    connect(reply,SIGNAL(uploadProgress(qint64,qint64)),this,SLOT(reply_uploadProgress(qint64,qint64)));
}

void WebRequest::head()
{
    QNetworkReply *reply = QNetworkAccessManager::head(request);
    connect(reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(reply_downloadProgress(qint64,qint64)));
    connect(reply,SIGNAL(error(QNetworkReply::NetworkError)),this,SLOT(reply_error(QNetworkReply::NetworkError)));
    connect(reply,SIGNAL(metaDataChanged()),this,SLOT(reply_metaDataChanged()));
    connect(reply,SIGNAL(sslErrors(QList<QSslError>)),this,SLOT(reply_sslErrors(QList<QSslError>)));
    connect(reply,SIGNAL(uploadProgress(qint64,qint64)),this,SLOT(reply_uploadProgress(qint64,qint64)));
}

void WebRequest::put()
{
    QNetworkReply *reply = QNetworkAccessManager::put(request,reqData);
    connect(reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(reply_downloadProgress(qint64,qint64)));
    connect(reply,SIGNAL(error(QNetworkReply::NetworkError)),this,SLOT(reply_error(QNetworkReply::NetworkError)));
    connect(reply,SIGNAL(metaDataChanged()),this,SLOT(reply_metaDataChanged()));
    connect(reply,SIGNAL(sslErrors(QList<QSslError>)),this,SLOT(reply_sslErrors(QList<QSslError>)));
    connect(reply,SIGNAL(uploadProgress(qint64,qint64)),this,SLOT(reply_uploadProgress(qint64,qint64)));
}

void WebRequest::reply_downloadProgress(qint64 recieved, qint64 total)
{
    emit downloaded(recieved,total);
}

void WebRequest::reply_error(QNetworkReply::NetworkError err)
{
    Q_UNUSED(err);
}

void WebRequest::reply_finished()
{
}

void WebRequest::reply_metaDataChanged()
{
}

void WebRequest::reply_sslErrors(const QList<QSslError> &errs)
{
    Q_UNUSED(errs);
}

void WebRequest::reply_uploadProgress(qint64 sent, qint64 total)
{
    emit uploaded(sent,total);
}

WebGetRequest::WebGetRequest(
        const QUrl &url,
        const QString &filePath)
        : WebRequest(url,filePath)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
}

WebGetRequest::WebGetRequest(
        const QUrl &url,
        const QString &aut,
        const QString &filePath)
        : WebRequest(url,filePath)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
    request.setRawHeader("Authorization",aut.toAscii());
}

WebGetRequest::~WebGetRequest()
{
}

void WebGetRequest::run()
{
    get();
}

WebPostRequest::WebPostRequest(
        const QUrl &url,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
    reqData = data;
}

WebPostRequest::WebPostRequest(
        const QUrl &url,
        const WebHeadersList &headers,
        const QString &contentType,
        int contentLength,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,contentType);
    request.setHeader(QNetworkRequest::ContentLengthHeader,contentLength);
    foreach(WebHeader h, headers)
        request.setRawHeader(h.first,h.second);
    reqData = data;
}

WebPostRequest::WebPostRequest(
        const QUrl &url,
        const WebHeadersList &headers,
        const QString &contentType,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,contentType);
    foreach(WebHeader h, headers)
        request.setRawHeader(h.first,h.second);
    reqData = data;
}

WebPostRequest::WebPostRequest(
        const QUrl &url,
        const WebHeadersList &headers,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    foreach(WebHeader h, headers)
        request.setRawHeader(h.first,h.second);
    reqData = data;
}

WebPostRequest::WebPostRequest(
        const QUrl &url,
        const QString &contentType,
        const QString &aut,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,contentType);
    request.setRawHeader("Authorization",aut.toAscii());
    reqData = data;
}

WebPostRequest::WebPostRequest(
        const QUrl &url,
        const QString &contentType,
        int contentLength,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,contentType);
    request.setHeader(QNetworkRequest::ContentLengthHeader,contentLength);
    reqData = data;
}

WebPostRequest::~WebPostRequest()
{
}

void WebPostRequest::run()
{
    post();
}

WebPutRequest::WebPutRequest(
        const QUrl &url,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");    
    reqData = data;
}

WebPutRequest::WebPutRequest(
        const QUrl &url,
        const WebHeadersList &headers,
        const QString &contentType,
        int contentLength,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,contentType);
    request.setHeader(QNetworkRequest::ContentLengthHeader,contentLength);
    reqData = data;
    foreach(WebHeader h, headers)
        request.setRawHeader(h.first,h.second);
}

WebPutRequest::WebPutRequest(
        const QUrl &url,
        const QString &contentType,
        const QString &aut,
        const QByteArray &data,
        const QString &fileName) : WebRequest(url,fileName)
{
    request.setHeader(QNetworkRequest::ContentTypeHeader,contentType);
    request.setRawHeader("Authorization",aut.toAscii());
    reqData = data;
}

WebPutRequest::~WebPutRequest()
{
}

void WebPutRequest::run()
{
    put();
}

WebHeadRequest::WebHeadRequest
        (const QUrl &url,
         const QString &fileName) : WebRequest(url, fileName)
{
}

WebHeadRequest::WebHeadRequest(
        const QUrl &url,
        const QString &fileName,
        const WebHeadersList &headers) : WebRequest(url, fileName)
{
    foreach(WebHeader h, headers)
        request.setRawHeader(h.first,h.second);
}

WebHeadRequest::~WebHeadRequest()
{
}

void WebHeadRequest::run()
{
    head();
}
