#ifndef PA_HTTP_H
#define PA_HTTP_H

#include "pA_Web_global.h"

#include "crypto/CP_RSA.h"
#include "crypto/base64.h"
#include "crypto/urlencode.h"
#include "crypto/md5.h"

#include <pa_photodata.h>

typedef QPair<QByteArray,QByteArray> WebHeader;
typedef QList<WebHeader> WebHeadersList;

struct WebReply
{
    struct Attributes
    {
        int statusCode;
        QByteArray reasonPhrase;
        QUrl redirectionTarget;
        bool connectionEncrypted;
        bool sourceIsFromCache;
        QString errorString;
        QNetworkReply::NetworkError error;
        QMap<QByteArray,QByteArray> rawHeaders;
    };
    Attributes attributes;
    QByteArray data;
    QString filePath;
};

class PA_WEBSHARED_EXPORT WebRequest : public QNetworkAccessManager
{
    Q_OBJECT

protected:
    QNetworkRequest request;
    QFile *respFile;    
    QByteArray reqData;
    QByteArray path;
public:
    WebRequest(const QUrl &url, const QString &filePath);
    ~WebRequest();
protected slots:
    void slot_autenticationRequired(QNetworkReply*, QAuthenticator*);
    void slot_finished(QNetworkReply*);
    void slot_proxyAuthenticationRequired(const QNetworkProxy&, QAuthenticator*);
    void slot_sslErrors(QNetworkReply*,const QList<QSslError>&);
    void reply_downloadProgress(qint64 recieved, qint64 total);
    void reply_error(QNetworkReply::NetworkError code);
    void reply_finished();
    void reply_metaDataChanged();
    void reply_sslErrors(const QList<QSslError> &errors);
    void reply_uploadProgress(qint64 sent, qint64 total);
signals:
    void done(const WebReply &reply);
    void offline();
    void proxyAuthentificationRequired();
    void uploaded(qint64 sent, qint64 total);
    void downloaded(qint64 recieced, qint64 total);
public slots:
    virtual void get();
    virtual void put();
    virtual void post();
    virtual void head();
public:
    virtual void run() = 0;
};

class PA_WEBSHARED_EXPORT WebGetRequest : public WebRequest
{
    Q_OBJECT

public:
    WebGetRequest(const QUrl &url, const QString &filePath);
    WebGetRequest(const QUrl &url, const QString &aut, const QString &filePath);
    WebGetRequest(const QUrl &url, const WebHeadersList &headers, const QString &filePath);
    virtual ~WebGetRequest();
public slots:
    virtual void run();
};

class PA_WEBSHARED_EXPORT WebPostRequest : public WebRequest
{
    Q_OBJECT

public:
    WebPostRequest(const QUrl &url, const QByteArray &data, const QString &fileName);
    WebPostRequest(const QUrl &url, const WebHeadersList &headers, const QString &contentType, int contentLength, const QByteArray &data, const QString &file);
    WebPostRequest(const QUrl &url, const QString &contentType, int contentLength, const QByteArray &data, const QString &fileName);
    WebPostRequest(const QUrl &url, const QString &contentType, const QString &aut, const QByteArray &data, const QString &fileName);
    WebPostRequest(const QUrl &url, const WebHeadersList &headers, const QByteArray &data, const QString &fileName);
    WebPostRequest(const QUrl &url, const WebHeadersList &headers, const QString &contentType, const QByteArray &data, const QString &file);
    virtual ~WebPostRequest();
public slots:
    virtual void run();
};

class PA_WEBSHARED_EXPORT WebPutRequest : public WebRequest
{
    Q_OBJECT

public:
    WebPutRequest(const QUrl &url, const QByteArray &data, const QString &fileName);
    WebPutRequest(const QUrl &url, const WebHeadersList &headers, const QString &contentType, int contentLength, const QByteArray &data, const QString &file);
    WebPutRequest(const QUrl &url, const QString &contentType, const QString &aut, const QByteArray &data, const QString &fileName);
    virtual ~WebPutRequest();
 public slots:
    virtual void run();
};

class PA_WEBSHARED_EXPORT WebHeadRequest : public WebRequest
{
    Q_OBJECT

public:
    WebHeadRequest(const QUrl &url, const QString &fileName);
    WebHeadRequest(const QUrl &url, const QString &fileName, const WebHeadersList &headers);
    virtual ~WebHeadRequest();
public slots:
    virtual void run();
};

#endif /* PA_HTTP_H */
